/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is LightSMS.
 *
 * The Initial Developer of the Original Code is
 * Byungwook Kang.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/***********************************************************
constants
***********************************************************/

const nsILightSMS = Components.interfaces.nsILightSMS;

// UUID uniquely identifying our component
const CLASS_ID = Components.ID("{20fd7fe8-248d-11dc-8314-0800200c9a66}");

// description
const CLASS_NAME = "LightSMS Service";

// textual unique identifier
const CONTRACT_ID = "@mozilla.org/LightSMS;1";

const DATA_VASP_ID=0;
const DATA_VAS_ID=1;
const DATA_USER_KEY=2;
const DATA_USER_LEVEL=3;
const DATA_USER_MDN=4;
const DATA_USER_NUM=5;
const DATA_FREE_COUNT=6;
const DATA_CASH_KEY=7;
const DATA_COUPON_COUNT=8;
const DATA_REFERER=9;

const STAGE_READY=20;
const STAGE_SEND=21;

const CMD_LOGIN=0;
const CMD_SEND=1;

const RETRY_LIMIT=3;

const dout=Components.utils.reportError;

/***********************************************************
class definition
***********************************************************/

//class constructor
function LightSMS() {
  var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                              .getService(Components.interfaces.nsIPrefService);
  this.branch = prefService.getBranch("lightsms.");
  this.branch.QueryInterface(Components.interfaces.nsIPrefBranch2);
  this.branch.addObserver("", this, false);

  var dir = Components.stack.toString();
  dir=dir.match(/:: (.+\/).+?/);
  dir=dir[1];
  var loader =  Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
                  .getService(Components.interfaces.mozIJSSubScriptLoader);
  loader.loadSubScript(dir+"SMS_Tworld.js");
  loader.loadSubScript(dir+"SMS_SHOW.js");
  loader.loadSubScript(dir+"SMS_Paran.js");
  loader.loadSubScript(dir+"SMS_NateOn.js");

  this.initHandler();

}

// class definition
LightSMS.prototype = {
  userData:[],
  listeners : new Array(),
  stage:0,
  onreq:false,
  cmd:CMD_LOGIN,
  recv:null,
  msg:null,
  retry:0,
  initHandler: function(){
    if(this.timer)this.timer.cancel();
    if(this.handler)this.handler.stop();
    var s=this.getPref("username");
    s=s.split("@");
    if(s[1]=="tworld.co.kr"){
      this.handler=new SMS_Tworld();
    }else if(s[1]=="show.co.kr"){
      this.handler=new SMS_SHOW();
    }else if(s[1]=="paran.com"){
      this.handler=new SMS_Paran();
    }else{
      this.handler=new SMS_NateOn();
    }
    this.handler.main=this;
    this.userData=[];
    this.userData[DATA_COUPON_COUNT]=0;
    this.retry=0;
    this.setState(nsILightSMS.ST_LOGIN,nsILightSMS.RS_TRY);
    this.handler.doLogin(CMD_LOGIN);
  },
  sendSMS: function(recv,msg){
    this.recv=recv;
    this.msg=msg;
    if(this.timer)this.timer.cancel();
    this.handler.stop();
    this.retry=0;
    if(!this.handler.loggedIn){
      this.setState(nsILightSMS.ST_LOGIN,nsILightSMS.RS_TRY);
      this.handler.doLogin(CMD_SEND);
    }else this.handler.send();
  },
  QueryInterface: function(aIID)
  {
    if (!aIID.equals(nsILightSMS) &&
        !aIID.equals(Components.interfaces.nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};
LightSMS.prototype.getPref=function(name){
  return this.branch.getCharPref(name);
}
LightSMS.prototype.setPref=function(name,value){
  this.branch.setCharPref(name,value);
}
LightSMS.prototype.addListener = function(aListener) {
  this.listeners.push(aListener);
  if(this.onreq)return;
  if(!this.handler.loggedIn){//not logged in
    this.retry=0;
    this.setState(nsILightSMS.ST_LOGIN,nsILightSMS.RS_TRY);
    this.handler.doLogin(CMD_LOGIN);
  }else{
    aListener.onStateChange(nsILightSMS.ST_LOGIN,nsILightSMS.RS_SUCC);
    aListener.onStateChange(nsILightSMS.ST_FREECOUNT,this.userData[DATA_FREE_COUNT]+this.userData[DATA_COUPON_COUNT]);
  }
}
LightSMS.prototype.removeListener = function(aListener){
  for(var i=0;i<this.listeners.length;i++){
    if(this.listeners[i]==aListener){
        this.listeners.splice(i,1);
        break;
    }
  }
}
LightSMS.prototype.setState = function(aID,aData) {
  for(var i=0;i<this.listeners.length;i++){
    if(this.listeners[i]){
      this.listeners[i].onStateChange(aID,aData);
    }
  }
}
LightSMS.prototype.showError = function(aData, aStatus,aHttp) {
  this.onreq=false;
  this.setState(nsILightSMS.ST_LOGIN,nsILightSMS.RS_FAIL);
dout("Err "+this.stage+" "+this.retry+" "+aStatus+"|"+aData);
  if (this.timer) {
    this.timer.cancel();
  } else {
    this.timer = Components.classes["@mozilla.org/timer;1"]
                  .createInstance(Components.interfaces.nsITimer);
  }
  try{
    this.timer.initWithCallback(this,
                                3000,
                                Components.interfaces.nsITimer.TYPE_ONE_SHOT);
  } catch (e) {
  }
}
LightSMS.prototype.notify = function(aTimer) {
//dout("reload "+this.retry);
  if(this.onreq||this.retry>RETRY_LIMIT)return;
  if(!this.handler.loggedIn){
    ++this.retry;
    this.setState(nsILightSMS.ST_LOGIN,nsILightSMS.RS_RETRY);
    this.handler.doLogin(this.cmd);
  }
}
/***********************************************************
preference observer
***********************************************************/
LightSMS.prototype.observe=function(aSubject, aTopic, aData)
{
  switch(aTopic){
  case "nsPref:changed":
    switch (aData) {
    case "showStatusbarIcon":
      this.setState(nsILightSMS.ST_ICON,this.branch.getBoolPref("showStatusbarIcon"));
      break;
    }
    break;
  }
}

/***********************************************************
password manager
***********************************************************/
LightSMS.prototype.getPassword=function(username){
  var srvc = Components.classes["@mozilla.org/autocomplete/search;1?name=password"]
                    .getService (Components.interfaces.acIPassword);
  return srvc.getPassword("chrome://lightsms/username",username);
}
LightSMS.prototype.getNumber=function(str){
  return str.replace(/[-\(\)]/g,"").match(/\d{3,}/);
}
LightSMS.prototype.getDateString=function(){
      var dt=new Date();
      function getString(n){
        if(n<10)return "0"+n;
        else return n;
      }
      return dt.getFullYear()+"-"+getString(dt.getMonth()+1)+"-"+getString(dt.getDate())
              +" "+getString(dt.getHours())+":"+getString(dt.getMinutes())+":"+getString(dt.getSeconds());

}
LightSMS.prototype.onSendSucc=function(){
  this.setState(nsILightSMS.ST_SEND,nsILightSMS.RS_SUCC);
  if(this.userData[DATA_FREE_COUNT]>0)--this.userData[DATA_FREE_COUNT];
  else if(this.userData[DATA_COUPON_COUNT]>0)--this.userData[DATA_COUPON_COUNT];
  if(this.branch.getBoolPref("saveHist")){
    var str=this.msg+"\t"+this.recv+"\t"+this.getPref("sender")+"\t"+this.getDateString();
    var db = Components.classes["@mozilla.org/autocomplete/search;1?name=text-file"]
                    .getService (Components.interfaces.nsITextFile);
    db.addEntry("smshist.txt", str,true);
  }
}


/***********************************************************
class factory

This object is a member of the global-scope Components.classes.
It is keyed off of the contract ID.
***********************************************************/
var LightSMSFactory = {
  createInstance: function (aOuter, aIID)
  {
    if (aOuter != null)
      throw Components.results.NS_ERROR_NO_AGGREGATION;
    return (new LightSMS()).QueryInterface(aIID);
  }
};

/***********************************************************
module definition (xpcom registration)
***********************************************************/
var LightSMSModule = {
  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
  {
    aCompMgr = aCompMgr.
        QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME,
        CONTRACT_ID, aFileSpec, aLocation, aType);
  },

  unregisterSelf: function(aCompMgr, aLocation, aType)
  {
    aCompMgr = aCompMgr.
        QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.unregisterFactoryLocation(CLASS_ID, aLocation);
  },

  getClassObject: function(aCompMgr, aCID, aIID)
  {
    if (!aIID.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (aCID.equals(CLASS_ID))
      return LightSMSFactory;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },
  canUnload: function(aCompMgr) { return true; }
};

/***********************************************************
module initialization

When the application registers the component, this function
is called.
***********************************************************/
function NSGetModule(aCompMgr, aFileSpec) { return LightSMSModule; }


/***********************************************************
HTMLReader
***********************************************************/
function HTMLReader(aCharSet) {
  if(aCharSet)this.charSet=aCharSet;
}
HTMLReader.prototype.getHtml = function(aCallback,aURL,aPostData,aHeaders) {
  var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
  var uri = ioService.newURI(aURL, "ISO-8859-1", null);
  var channel = ioService.newChannelFromURI(uri);
  var httpChannel = channel.QueryInterface(Components.interfaces.nsIHttpChannel);
  if(aHeaders){
    for(var t in aHeaders){
      httpChannel.setRequestHeader(t,aHeaders[t],false);
    }
  }
  httpChannel.setRequestHeader("User-Agent","Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0)",false);
  if (aPostData) {
    var uploadStream = Components.classes["@mozilla.org/io/string-input-stream;1"]
      .createInstance(Components.interfaces.nsIStringInputStream);
    var uploadChannel = channel.QueryInterface(Components.interfaces.nsIUploadChannel);
    if(typeof aPostData =="string"){
      uploadStream.setData(aPostData, aPostData.length);
      uploadChannel.setUploadStream(uploadStream, "application/x-www-form-urlencoded", -1);
    }else{//multipart
      var boundary="---------------------------";
      for(var i=0;i<3;i++){
        var t=parseInt(Math.random()*0x7fff);
        boundary+=t.toString(16);
      }
      var post="";
      var uniConv = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"]
  		  							.createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
      uniConv.charset = "euc-kr";
      httpChannel.setRequestHeader("Accept-Language","ko",false);
      for(var i in aPostData){
        post+="--"+boundary+"\r\n"+"Content-Disposition: form-data; name=\""+i+"\""
              +"\r\n\r\n"+uniConv.ConvertFromUnicode(aPostData[i])+"\r\n";
      }
      post+="--"+boundary+"--\r\n";
      uploadStream.setData(post, post.length);
      uploadChannel.setUploadStream(uploadStream, "multipart/form-data; boundary="+boundary, -1);
    }
    httpChannel.requestMethod = "POST";
  }
  this.callback=aCallback;
  this.httpChannel=httpChannel;
  channel.asyncOpen(this,httpChannel);
}
HTMLReader.prototype.onStartRequest = function (aRequest, aContext) {
  this.data = "";
}
HTMLReader.prototype.onDataAvailable = function (aRequest, aContext, aStream, aSourceOffset, aLength){
  var is = Components.classes["@mozilla.org/scriptableinputstream;1"]
            .createInstance(Components.interfaces.nsIScriptableInputStream);
  is.init(aStream);
  this.data += is.read(aLength);
}

HTMLReader.prototype.onStopRequest = function (aRequest, aContext, aStatus) {
  this.httpChannel=null;
  var response=-1;
  try{
    response=aContext.QueryInterface(Components.interfaces.nsIHttpChannel).responseStatus
  }catch(e){}

  var charSet=null;
  var httpChannel=aContext.QueryInterface(Components.interfaces.nsIHttpChannel);
  if(this.charSet)charSet=this.charSet;
  else{
    var fnd=this.data.match(/<meta.+?\".+?charset=(\S+?)\s*?\"\s*?\x2f*>/);
    if(fnd)charSet=fnd[1];
    else{
      try{
        if(httpChannel.contentCharset)charSet=httpChannel.contentCharset;
      }catch(e){}
    }
  }
  if(charSet){
    var uniConv = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"]
    									.createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
    uniConv.charset = charSet;
    try{
      this.data=uniConv.ConvertToUnicode(this.data);
    }catch(e){}
  }
  this.callback.callback(this.data,response,aContext);
}
HTMLReader.prototype.stop = function () {
  if(this.httpChannel){
    this.httpChannel.cancel(Components.results.NS_BINDING_ABORTED);
    this.httpChannel=null;
  }
}

function SocketReader(callback,host,port) {
  this.callback=callback;
  var transportService =
    Components.classes["@mozilla.org/network/socket-transport-service;1"]
      .getService(Components.interfaces.nsISocketTransportService);
  this.transport = transportService.createTransport(null,0,host,port,null);
  this.outstream = this.transport.openOutputStream(0,0,0);
  var stream = this.transport.openInputStream(0,0,0);
  this.instream = Components.classes["@mozilla.org/scriptableinputstream;1"]
    .createInstance(Components.interfaces.nsIScriptableInputStream);
  this.instream.init(stream);
  var pump = Components.
    classes["@mozilla.org/network/input-stream-pump;1"].
      createInstance(Components.interfaces.nsIInputStreamPump);
  pump.init(stream, -1, -1, 0, 0, false);
  pump.asyncRead(this,null);
}
SocketReader.prototype.write = function (data) {
  this.outstream.write(data,data.length);
}
SocketReader.prototype.onStartRequest = function (aRequest, aContext) {
}
SocketReader.prototype.onDataAvailable = function (aRequest, aContext, aStream, aSourceOffset, aLength){
  this.callback.callback(this.instream.read(aLength),1);
}

SocketReader.prototype.onStopRequest = function (aRequest, aContext, aStatus) {
  this.close();
  //this.callback.callback("",0);
}
SocketReader.prototype.close = function () {
  this.instream.close();
  this.outstream.close();
}

function SMSHandler(){
  this.reader=new HTMLReader("euc-kr");
  this.loggedIn=false;
}

SMSHandler.prototype.doLogin = function(aCmd) {
  this.main.cmd=aCmd;
  this.main.onreq=true;
  this.main.stage=0;
}
SMSHandler.prototype.stop = function() {
  this.reader.stop();
}
SMSHandler.prototype.loginSucc = function(freeCount) {
  this.main.setState(nsILightSMS.ST_LOGIN,nsILightSMS.RS_SUCC);
  this.main.setState(nsILightSMS.ST_FREECOUNT,freeCount);
  this.main.stage=STAGE_READY;
  if(this.main.cmd==CMD_SEND){
    this.send();
  }else this.main.onreq=false;
}
SMSHandler.prototype.sendResult = function(fnd) {
  this.main.onreq=false;
  if(fnd){
    this.main.onSendSucc();
    return true;
  }else{
    if(this.main.retry<RETRY_LIMIT){
      ++this.main.retry;
      this.main.setState(nsILightSMS.ST_LOGIN,nsILightSMS.RS_TRY);
      this.loggedIn=false;
      this.doLogin(CMD_SEND);
      return false;
    }else{
      this.main.setState(nsILightSMS.ST_SEND,nsILightSMS.RS_FAIL);
    }
    return false;
  }
}